# -*- coding: utf-8 -*-

import bpy, os, sys

scene_name  = 'Scene'
screen_name = 'Default'
area_type   = 'VIEW_3D'
space_type  = 'VIEW_3D'

use_modifiers = False		# not used....


def exportply( obj, filepath ):
	mesh = obj.data;
	mesh_verts = mesh.vertices
	mesh_edges = mesh.edges
	mesh_faces = mesh.faces
	
	# (mesh.total_vert_sel,mesh.total_edge_sel,mesh.total_face_sel)
	# these are not updated when running in background mode!!!!!
	nselverts = 0
	nseledges = 0
	nselfaces = 0
	for v in mesh_verts:
		if( v.select ):
			nselverts += 1
	for e in mesh_edges:
		if( e.select ):
			nseledges += 1
	for f in mesh_faces:
		if( f.select ):
			nselfaces += 1
	
	#if scene.objects.active:
	#	bpy.ops.object.mode_set(mode='OBJECT')
	#if use_modifiers:
	#	mesh = obj.create_mesh(scene, True, 'PREVIEW')
	#else:
	#	mesh = obj.data
	
	file = open(filepath, 'w')
	file.write('ply\n')
	file.write('format ascii 1.0\n')
	file.write('comment Modified PLY\n')
	file.write('comment Created by Blender %s - www.blender.org, source file: %r\n' % (bpy.app.version_string, os.path.basename(bpy.data.filepath)))
	
	file.write('element vertex %d\n' % len(mesh_verts))
	file.write('element face %d\n' % (len(mesh.faces)+len(mesh_edges)))
	file.write('element selected %d %d %d\n' % (nselverts,nseledges,nselfaces))
	file.write('element modifiers %d\n' % len(obj.modifiers))
	
	file.write('end_header\n')
	
	# Print out locations of vertices
	for v in mesh_verts:
		tv = obj.matrix_world * v.co
		file.write( '%.6f %.6f %.6f ' % tuple(tv) )
		file.write( '%d\n' % (not v.hide) )
	
	# Print out indices of edges
	for e in mesh_edges:
		file.write( '2 %d %d %d %d\n' % ( e.vertices[0], e.vertices[1], not e.hide, e.select ) )
	
	# Print out indices of faces
	for f in mesh_faces:
		if len(f.vertices) == 3:
			file.write( '3 %d %d %d %d %d\n' % (f.vertices[0],f.vertices[1],f.vertices[2],not f.hide, f.select) )
		else:
			file.write( '4 %d %d %d %d %d %d\n' % (f.vertices[0],f.vertices[1],f.vertices[2],f.vertices[3],not f.hide, f.select) )
	
	# Print out indices of selected vertices
	for v in mesh_verts:
		if v.select:
			file.write('%d\n' % v.index)
	
	# Print out modifiers
	# bpy.ops.object.modifier_apply
	for m in obj.modifiers:
		if( m.type == 'MIRROR' ):
			file.write( 'mirror "%s" %d %d %d %.6f\n' % (m.name, m.use_x, m.use_y, m.use_z, m.merge_threshold) )
		elif( m.type == 'SUBSURF' ):
			file.write( 'subdiv "%s" %d\n' % (m.name, m.levels) )
		elif( m.type == 'BOOLEAN' ):
			file.write( 'boolop "%s" %s %s\n' % (m.name, m.operation, m.object.name) )
		elif( m.type == 'SOLIDIFY' ):
			file.write( 'solidify "%s" %.6f %.6f ' % (m.name, m.thickness, m.offset) )
			file.write( '%.6f %.6f %.6f ' % ( m.edge_crease_inner, m.edge_crease_outer, edge_crease_rim) )
			file.write( '%d %d %d \n' % (m.use_even_offset, m.use_quality_normals, m.use_rim) )
	
	file.close()




scene = bpy.data.scenes[scene_name]
if not scene:
	raise Exception("Error, Cannot find 'Scene' scene")

screen = bpy.data.screens[screen_name]
if not screen:
	raise Exception("Error, Cannot find 'Default' screen")

# count number of views
area_count = 0
for a in screen.areas:
	if a.type == area_type:
		area_count += 1
obj_count = 0
for o in scene.objects:
	if o.type == 'MESH':
		obj_count += 1

#####################################################
# Start writing to .ply files

filepath = "%s.ply" % (bpy.data.filepath)
file = open(filepath, 'w')
file.write('ply\n')
file.write('format ascii 1.0\n')
file.write('comment Modified PLY\n')
file.write('comment Created by Blender %s - www.blender.org, source file: %r\n' % (bpy.app.version_string, os.path.basename(bpy.data.filepath)))
file.write('element views %d\n' % area_count)
file.write('element objects %d\n' % obj_count)
file.write('end_header\n')

# Print out current view for each area
for a in screen.areas:
	if a.type != area_type:
		continue
	
	for s in a.spaces:
		if s.type == space_type:
			space = s
			break
	if not space:
		raise Exception("Error, could not find VIEW_3D space")
	
	region = space.region_3d
	
	file.write( '%.6f %.6f %.6f ' % tuple(region.view_location) )
	file.write( '%.6f %.6f %.6f %.6f ' % tuple(region.view_rotation) )
	file.write( '%.6f ' % (region.view_distance) )
	file.write( '%s\n' % (region.view_perspective) )

for obj in scene.objects:
	if obj.type != 'MESH':
		continue
	indx = scene.objects.values().index(obj)
	filepath = "%s.%02d.ply" % (bpy.data.filepath, indx)
	isselected = obj in bpy.context.selected_editable_objects
	isactive = (bpy.context.active_object == obj)
	isedit = (obj.mode == 'EDIT')
	isvisible = (not obj.hide)
	file.write('"%s" %d %d %d %d %s\n' % (obj.name, isvisible, isselected, isactive, isedit, filepath) )
	#isedit = (bpy.context.edit_object == obj)
	#file.write('"%s" %d %d %d %s\n' % (obj.name, isselected, isactive, isedit, filepath) )
	exportply( obj, filepath )

file.close()





#####################################################
# Report to Console

print("writing %r done" % filepath)


#####################################################
# "Undo" modifier preview...

if use_modifiers:
	bpy.data.meshes.remove(mesh)


#####################################################
# Done!

#quit()
